@@ -9,6 +9,8 @@ if TYPE_CHECKING:
     from xdsl.parser import Parser
     from xdsl.printer import Printer
 
+OperationType = TypeVar('OperationType', bound='Operation', covariant=True)
+
 
 @dataclass
 class MLContext:
@@ -50,6 +52,19 @@ class SSAValue(ABC):
     typ: Attribute
     """Each SSA variable is associated to a type."""
 
+    @staticmethod
+    def get(arg: SSAValue | Operation) -> SSAValue:
+        """Get a new SSAValue from either a SSAValue, or an operation with a single result."""
+        if isinstance(arg, SSAValue):
+            return arg
+        if isinstance(arg, Operation):
+            if len(arg.results) == 1:
+                return arg.results[0]
+            raise ValueError(
+                "SSAValue.build: expected operation with a single result.")
+        raise TypeError(
+            f"Expected SSAValue or Operation for SSAValue.get, but got {arg}")
+
 
 @dataclass
 class OpResult(SSAValue):
@@ -60,6 +75,7 @@ class OpResult(SSAValue):
 
     result_index: int
     """The index of the result in the defining operation."""
+
     def __hash__(self):
         return hash((id(self.op), self.result_index))
 
@@ -73,10 +89,14 @@ class BlockArgument(SSAValue):
 
     index: int
     """The index of the variable in the block arguments."""
+
     def __hash__(self):
         return hash((id(self.block), self.index))
 
 
+AttrClass = TypeVar('AttrClass', bound='Attribute')
+
+
 @dataclass(frozen=True)
 class Attribute(ABC):
     """
@@ -88,10 +108,16 @@ class Attribute(ABC):
     name: str = field(default="", init=False)
     """The attribute name should be a static field in the attribute classes."""
 
+    @classmethod
+    def build(cls: typing.Type[AttrClass], *args) -> AttrClass:
+        """Create a new attribute using one of the builder defined in IRDL."""
+        assert False
+
 
 @dataclass(frozen=True)
 class Data(Attribute):
     """An attribute represented by a Python structure."""
+
     @staticmethod
     @abstractmethod
     def parse(parser: Parser) -> Data:
@@ -118,11 +144,8 @@ class ParametrizedAttribute(Attribute):
         ...
 
 
-T = TypeVar('T', bound='Operation')
-
-
 @dataclass
-class Operation(ABC):
+class Operation:
     """A generic operation. Operation definitions inherit this class."""
 
     name: str = field(default="", init=False)
@@ -148,6 +171,7 @@ class Operation(ABC):
 
     parent: Optional[Block] = field(default=None)
     """The block containing this operation."""
+
     def __post_init__(self):
         assert (self.name != "")
         assert (isinstance(self.name, str))
@@ -159,8 +183,12 @@ class Operation(ABC):
                           attributes: Optional[Dict[str, Attribute]] = None,
                           successors: Optional[List[Block]] = None,
                           regions: Optional[List[Region]] = None) -> Operation:
+
         operation = op()
         if operands is not None:
+            for operand in operands:
+                assert isinstance(
+                    operand, SSAValue), "Operands must be of type SSAValue"
             operation.operands = operands
         if result_types is not None:
             operation.results = [
@@ -177,15 +205,25 @@ class Operation(ABC):
         return operation
 
     @classmethod
-    def create(cls: typing.Type[T],
+    def create(cls: typing.Type[OperationType],
                operands: Optional[List[SSAValue]] = None,
                result_types: Optional[List[Attribute]] = None,
                attributes: Optional[Dict[str, Attribute]] = None,
                successors: Optional[List[Block]] = None,
-               regions: Optional[List[Region]] = None) -> T:
+               regions: Optional[List[Region]] = None) -> OperationType:
         return Operation.with_result_types(cls, operands, result_types,
                                            attributes, successors, regions)
 
+    @classmethod
+    def build(cls: typing.Type[T],
+              operands=[],
+              result_types=[],
+              attributes=[],
+              successors=[],
+              regions=[]) -> T:
+        """Create a new operation using builders."""
+        ...
+
     def add_region(self, region: Region) -> None:
         self.regions.append(region)
         region.parent = self
@@ -201,9 +239,8 @@ class Operation(ABC):
         for region in self.regions:
             region.verify()
 
-    @abstractmethod
     def verify_(self) -> None:
-        ...
+        pass
 
     def __eq__(self, other: Operation) -> bool:
         return self is other
@@ -224,14 +261,33 @@ class Block:
 
     parent: Optional[Region] = field(default=None)
     """Parent region containing the block."""
+
     @staticmethod
-    def with_arg_types(arg_types: List[Attribute]) -> Block:
+    def from_arg_types(arg_types: List[Attribute]) -> Block:
         b = Block()
         b.args = [
             BlockArgument(typ, b, index) for index, typ in enumerate(arg_types)
         ]
         return b
 
+    @staticmethod
+    def from_ops(ops: List[Operation], arg_types: List[Attribute] = None):
+        b = Block()
+        if arg_types is not None:
+            b.args = [
+                BlockArgument(typ, b, index)
+                for index, typ in enumerate(arg_types)
+            ]
+        b.add_ops(ops)
+        return b
+
+    @staticmethod
+    def from_callable(block_arg_types: List[Attribute],
+                      f: Callable[[Attribute, ...], List[Operation]]):
+        b = Block.from_arg_types(block_arg_types)
+        b.add_ops(f(*b.args))
+        return b
+
     def add_op(self, operation: Operation) -> None:
         self.ops.append(operation)
         operation.parent = self
@@ -265,6 +321,7 @@ class Region:
 
     parent: Optional[Operation] = field(default=None)
     """Operation containing the region."""
+
     @staticmethod
     def from_operation_list(ops: List[Operation]) -> Region:
         block = Block([], ops)
@@ -272,6 +329,26 @@ class Region:
         region.add_block(block)
         return region
 
+    @staticmethod
+    def from_block_list(blocks: List[Block]) -> Region:
+        region = Region()
+        for block in blocks:
+            region.add_block(block)
+        return region
+
+    @staticmethod
+    def get(arg: Region | List[Block] | List[Operation]) -> Region:
+        if isinstance(arg, Region):
+            return arg
+        if isinstance(arg, list):
+            if len(arg) == 0:
+                raise ValueError("Can't construct a region with an empty list")
+            if isinstance(arg[0], Block):
+                return Region.from_block_list(arg)
+            if isinstance(arg[0], Operation):
+                return Region.from_operation_list(arg)
+        raise TypeError(f"Can't build a region with argument {arg}")
+
     def add_block(self, block: Block) -> None:
         self.blocks.append(block)
         block.parent = self
