@@ -525,6 +525,11 @@ OperationType = TypeVar("OperationType", bound=Operation)
 def irdl_op_definition(
         cls: typing.Type[OperationType]) -> typing.Type[OperationType]:
     """Decorator used on classes to define a new operation definition."""
+
+    assert issubclass(
+        cls,
+        Operation), f"class {cls.__name__} should be a subclass of Operation"
+
     operand_defs = [(field_name, field)
                     for field_name, field in cls.__dict__.items()
                     if isinstance(field, OperandDef)]
@@ -604,7 +609,7 @@ def irdl_op_definition(
 
     new_attrs["build"] = classmethod(builder)
 
-    return type(cls.__name__, (Operation, ), {**cls.__dict__, **new_attrs})
+    return type(cls.__name__, (cls, ), {**cls.__dict__, **new_attrs})
 
 
 @dataclass
