@@ -5,7 +5,7 @@ from dataclasses import dataclass, field
 from typing import List, Optional, Callable, Dict
 
 from xdsl.dialects.builtin import ModuleOp
-from xdsl.ir import Operation, SSAValue, Region, Block
+from xdsl.ir import Operation, SSAValue, Region, Block, BlockArgument
 
 
 @dataclass(eq=False, repr=False)
@@ -21,6 +21,7 @@ class RewriteAction:
 
     new_results: List[Optional[SSAValue]]
     """SSA values that replace the matched operation results. None values are deleted SSA Values."""
+
     @staticmethod
     def from_op_list(new_ops: List[Operation]) -> RewriteAction:
         """
@@ -37,6 +38,7 @@ class RewritePattern(ABC):
     """
     A side-effect free rewrite pattern matching on a DAG.
     """
+
     @abstractmethod
     def match_and_rewrite(
             self, op: Operation,
@@ -119,6 +121,7 @@ class GreedyRewritePatternApplier(RewritePattern):
 
     rewrite_patterns: List[RewritePattern]
     """The list of rewrites to apply in order."""
+
     def match_and_rewrite(
             self, op: Operation,
             new_operands: List[SSAValue]) -> Optional[RewriteAction]:
@@ -139,6 +142,7 @@ class OperandUpdater:
     result_mapping: Dict[SSAValue,
                          Optional[SSAValue]] = field(default_factory=dict)
     """Map old ssa values to new values. Deleted values are mapped to None."""
+
     def bookkeep_results(self, old_op: Operation,
                          action: RewriteAction) -> None:
         """Bookkeep the changes made by a rewrite action matching on `old_op`."""
@@ -150,6 +154,11 @@ class OperandUpdater:
         for (old_res, new_res) in zip(old_op.results, action.new_results):
             self.result_mapping[old_res] = new_res
 
+    def bookkeep_blockargs(self, old_block: Block, new_block: Block):
+        """ Map old block arguments to the new arguments """
+        for (old_arg, new_arg) in zip(old_block.args, new_block.args):
+            self.result_mapping[old_arg] = new_arg
+
     def get_new_value(self, value: SSAValue) -> Optional[SSAValue]:
         """Get the updated value, if it exists, or returns the same one."""
         return self.result_mapping.get(value, value)
@@ -187,6 +196,7 @@ class PatternRewriteWalker:
     _updater: OperandUpdater = field(init=False,
                                      default_factory=OperandUpdater)
     """Takes care of bookkeeping the changes made during the walk."""
+
     def rewrite_module(self, op: ModuleOp):
         """Rewrite an entire module operation."""
         new_ops = self.rewrite_op(op)
@@ -227,6 +237,11 @@ class PatternRewriteWalker:
             self.rewrite_op_regions(op)
         return [op]
 
+    def add_block_args(self, new: Block, old: Block):
+        """Duplicate the old BlockArguments, adds them to the new Block, and bookkeeps them."""
+        new.args = [BlockArgument(arg.typ, new, arg.index) for arg in old.args]
+        self._updater.bookkeep_blockargs(old, new)
+
     def rewrite_op_regions(self, op: Operation):
         """Rewrite the regions of an operation, and update the operation with the new regions."""
         new_regions = []
@@ -234,6 +249,7 @@ class PatternRewriteWalker:
             new_region = Region()
             for block in region.blocks:
                 new_block = Block()
+                self.add_block_args(new_block, block)
                 for sub_op in block.ops:
                     new_block.add_ops(self.rewrite_op(sub_op))
                 new_region.add_block(new_block)
